DropSliceMetadata

Struct DropSliceMetadata 

pub struct DropSliceMetadata { /* private fields */ }
The metadata associated with one payload slice decoded from data in the Willow drop format.

This metadata consists of the AuthorisedEntry of which the slice is a part, the chunk index at which the slice starts, the number of chunks in the slice, and the left_skip minimising the verification data required for this slice in the context of the drop.

The drop format APIs usually work with producers of payload slices, paired with a DropSliceMetadata. The entries that make up a drop are typically communicated only through the entry field of the DropSliceMetadata.

Implementations

Source

impl DropSliceMetadata

pub fn new( entry: AuthorisedEntry, first_chunk: u64, chunk_count: u64, left_skip: u8, is_multislice_encoded: bool, ) -> Result<Self, SliceMetadataError>

Creates a new DropSliceMetadata describing the 64-grouped baseline verifiable slice stream of the slice of entry from first_chunk to first_chunk + chunk_count.

pub fn entry(&self) -> &AuthorisedEntry

Returns the AuthorisedEntry to which the payload slice described by self contributes.

pub fn first_chunk(&self) -> u64

Returns in the index in the total payload of the first chunk of the payload slice described by self.

pub fn chunk_count(&self) -> u64

Returns the number of chunks in the payload slice described by self.

pub fn left_skip(&self) -> u8

Returns the left_skip used to encode the payload slice described by self.

pub fn resumption_info(&self) -> SliceStreamResumptionInfo

Returns the SliceStreamResumptionInfo indicating where the payload slice described by self can be appended to existing payload data.

pub fn streaming_options(&self) -> SliceStreamingOptions

Returns the SliceStreamingOptions used to encode the payload slice described by self.

pub fn expected_bytes(&self) -> u64

Returns the number of bytes which are expected to occur in the 64-grouped baseline verifiable stream of the payload slice described by self.

pub fn is_complete(&self) -> bool

Returns true if the payload slice described by self covers the complete payload of the entry, else false.

pub fn is_multislice(&self) -> bool

Returns true if the payload slice described by self might be one of multiple slices from the same entry in the drop, else false.