SliceStreamProducer

Struct SliceStreamProducer 

pub struct SliceStreamProducer<P>
where P: BulkProducer<Item = u8>,
{ /* private fields */ }
A BulkProducer<Item=u8> produced by a DropDecoder, which produces the verifiable slice stream for one contiguous slice of the payload of one AuthorisedEntry decoded from the Willow Drop Format.

Warning

Any SliceStreamProducer produced by a DropDecoder must be completely consumed, emitting its Final value in order for further calls to DropDecoder::produce to make progress.