SubspaceId

Struct SubspaceId 

pub struct SubspaceId(/* private fields */);
The type of SubspaceIds used by Willow’25.

Subspaces group Willow data within the same namespace into independent collections — entries with nonequal subspace ids cannot overwrite each other. Subspaces further play a role in access control for Willow: subspace ids are public keys of a digital signature system, and knowledge of the corresponding secret key allow you to issue write entries to a subspace, and to delegate access to other users — see the meadowcap module for more details.

That all said, you can also simply think of subspace ids as glorified [u8; 32] arrays. That perspective should be sufficient as long as you do not need to care about cryptographic details.

You will typically obtain a subspace id either by randomly generating a key pair, by cloning an existing subspace id, or by creating it from raw bytes via SubspaceId::from_bytes. See the Willow25 specification if you want to work with raw bytes — while every [u8; 32] can be converted into a SubspaceId, some of these do not correspond to canonic public key encodings and thus always fail signature verification.

use rand::rngs::OsRng;
use willow25::prelude::*;

let mut csprng = OsRng; // cryptographically secure pseudo-random number generator
let (_subspace_id, _secret) = randomly_generate_subspace(&mut csprng);

let subspace_id2 = SubspaceId::from_bytes(&[17; SUBSPACE_ID_WIDTH]);
assert_eq!(subspace_id2.as_bytes(), &[17; 32]);

Implementations

Source

impl SubspaceId

pub fn from_bytes(bytes: &[u8; 32]) -> Self

Returns a new SubspaceId corresponding to the supplied bytes.

use willow25::prelude::*;

let subspace_id = SubspaceId::from_bytes(&[17; SUBSPACE_ID_WIDTH]);
assert_eq!(subspace_id.as_bytes(), &[17; SUBSPACE_ID_WIDTH]);
pub fn as_bytes(&self) -> &[u8; 32]

Returns a reference to the raw bytes that make up this subspace id.

use willow25::prelude::*;

let subspace_id = SubspaceId::from_bytes(&[17; SUBSPACE_ID_WIDTH]);
assert_eq!(subspace_id.as_bytes(), &[17; SUBSPACE_ID_WIDTH]);
pub fn to_bytes(&self) -> [u8; 32]

Returns the raw bytes that make up this subspace id.

use willow25::prelude::*;

let subspace_id = SubspaceId::from_bytes(&[17; SUBSPACE_ID_WIDTH]);
assert_eq!(subspace_id.to_bytes(), [17; SUBSPACE_ID_WIDTH]);
pub fn from_ed25519_verifying_key(verifying_key: VerifyingKey) -> Self

Creates a subspace id from an ed25519 verifying key.

use willow25::prelude::*;

let from_key = SubspaceId::from_ed25519_verifying_key(
    ed25519_dalek::VerifyingKey::from_bytes(&[17; 32]).unwrap(),
);
let from_bytes = SubspaceId::from_bytes(&[17; SUBSPACE_ID_WIDTH]);

assert_eq!(from_key, from_bytes);