pub enum CreateEntryError<StoreError> {
StoreError(StoreError),
AuthorisationTokenError,
}Expand description
The errors that can be encountered when creating an entry (from a producer emitting the payload and the ingredients for authorising it).
Variants
StoreError(StoreError)
The store implementation encountered an internal error.
AuthorisationTokenError
Could not create an authorisation token for the created entry.