pub enum NondestructiveInsert {
Success(AuthorisedEntry),
Prevented,
Outdated,
}Expand description
The possible successful outcomes of non-destructively inserting an AuthorisedEntry into a Store.
Variants
Success(AuthorisedEntry)
Inserted the entry, and no other data was pruned.
Prevented
Inserting an entry would have pruned old entries, so instead the store was not modified at all.
Outdated
The inserted entry would be pruned by a newer entry in the store, so it was not inserted in the first place.