pub struct SubspaceSignature(/* private fields */);Expand description
A secure digital signature, issued by a SubspaceSecret and verifiable with the corresponding SubspaceId.
This type is a fairly thin wrapper around ed25519_dalek::Signature, use the From implementations to convert between the two.
Chances are you never need to interact with this type directly, the [meadowcap] module should handle all authentication needs for you.